// $Id: Actors.C 15422 2015-11-13 21:06:21Z igor $
// This is a GGS file, licensed under the GPL

/*
    (c) Igor Durdanovic, igord@research.nj.nec.com
    NEC Research Institute
    4 Independence Way
    Princeton, NJ 08540, USA

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
//: Actors.C (bof) (c) Igor Durdanovic

#include "Actors.H"
#include "Signal.H"
#include "Stat.H"
#include "IO_FILE.H"
#include "IO_Server.H"
#include "DB_Server.H"
#include "TIME_Server.H"
#include "TSTAT_Server.H"
#include "EXE_Client.H"
#include "VAR_System.H"
#include "Message.H"
#include "SET_PTR_Mux.H"

GAS::GAS()
  : Central_root_name ( "Igor Durdanovic" ),
    Central_root_email( "igord@nec-labs.com" ),

    root_level ( 3 ),
    admin_level( 2 ),
    user_level ( 1 ),
    unreg_level( 0 ),

    login_system( " s.y.s.t.e.m " ),
    login_root  ( "root" ),

    group_service( "_service" ), // all service providers
    group_client ( "_client" ),  // all clients
    group_admin  ( "_admin" ),   // admin group
    chann_help   ( ".help" ),    // help channel
    chann_chat   ( ".chat" ),    // chat channel
    
    var_lrecv    ( "_recv" ),      // who sent the last message we received
    var_lsend    ( "_send" ),      // to whom we sent the last message
    var_prefix   ( "_pfix" ),      // prefix all commands with ..
    var_rsec     ( "_rsec" ),      // repeat after sec
    var_rcmd     ( "_rcmd" ),      // repeat cmd 
    var_ready    ( "_ready" ),     // READY prompt
    var_alert    ( "_alert" ),     // ALERT prompt

    max_alias  ( 64 ),     // max no of aliases   per user
    max_vars   ( 64 ),     // max no of variables per user
    max_chann  ( 64 ),     // max no of channels  per user
    max_ignore ( 64 ),     // max no of ignoreds  per user
    max_notify ( 64 ),     // max no of notifys   per user
    max_str_len( 1 << 10 ), // 1K

    str_empty( "" ),
    str_plus ( "+" ),
    str_minus( "-" ),
    str_pct  ( "%" ),
    str_star ( "*" ),
    str_quest( "?" ),
    str_ready( "READY\r\n" ),
    str_alert( "ALERT\r\n" ),

    file_con( "log/Central.con" ),
    file_log( "log/Central.log" ),
    file_db (  "db/Central.db" ),

    time_log  ( 24*60*60 ),
    time_db   ( 24*60*60 ),
    time_stat (       60 ),
    time_logon(     2*60 ),

    warn_blocks( 1024 * 8 ),
    exit_blocks( 1024 * 2 ),
    warn_inodes( 1024 * 8 ),
    exit_inodes( 1024 * 2 ),
    warn_memory( 1024 * 1024 * 4 ),
    exit_memory( 1024 * 1024 ),
    srv_ready  ( true ),
    tstat_ready( false ),
    vc_trace( 0 ),

    vc_tstat(0),
    vc_con  (0),
    vc_sig  (0),
    vc_time (0),
    vc_db   (0),
    vc_ios  (0),
    vc_log  (0),
    vc_var  (0),
    vc_exe  (0),
    vc_stat (0),
    vc_mssg (0),
    vc_mux  (0)
{}

void GAS::init()
{
  vc_tstat = new TSTAT_Server;
  vc_con   = new IO_FILE( file_con, true,  false );
  vc_sig   = new Signal;                                // needs vc_con !!
  vc_time  = new TIME_Server;                           // needs vc_con, vc_log
  vc_db    = new DB_Server( file_db, IO_Buffer::NONE );   // needs vc_con, vc_log, vc_time
  vc_ios   = new IO_Server;                             // needs vc_sig, vc_con, vc_time, vc_mux
  vc_log   = new IO_FILE( file_log, false, false, false, time_log ); // needs vc_ios
  vc_var   = new VAR_System;                            // needs vc_con, vc_db
  vc_exe   = new EXE_Client;
  vc_stat  = new Stat;        // needs vc_time
  vc_mssg  = new Message;     // needs nothing
  vc_mux   = new SET_PTR_Mux; // needs nothing
}

GAS::~GAS()
{
  delete vc_mux  ; vc_mux   = 0;
  delete vc_mssg ; vc_mssg  = 0;
  delete vc_stat ; vc_stat  = 0;
  delete vc_exe  ; vc_exe   = 0;
  delete vc_var  ; vc_var   = 0;
  delete vc_log  ; vc_log   = 0;
  delete vc_ios  ; vc_ios   = 0;
  delete vc_db   ; vc_db    = 0;
  delete vc_time ; vc_time  = 0;
  delete vc_sig  ; vc_sig   = 0;
  delete vc_con  ; vc_con   = 0;
  delete vc_tstat; vc_tstat = 0;
}

std::unique_ptr< GAS > gas;

//: Actors.C (eof) (c) Igor Durdanovic
