// $Id: Actors.C 15423 2015-11-13 21:15:40Z igor $
// This is a GGS file, licensed under the GPL

/*
    (c) Igor Durdanovic, igord@research.nj.nec.com
    NEC Research Institute
    4 Independence Way
    Princeton, NJ 08540, USA

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
//: Actors.C (bof) (c) Igor Durdanovic

#include "Actors.H"
#include "Signal.H"
#include "Stat.H"
#include "IO_Server.H"
#include "IO_FILE.H"
#include "TIME_Server.H"
#include "IO_TCP_Mux.H"

GAS::GAS()
  : time_log ( 24*60*60 ),
    time_stat(       60 ),
    file_log ( "log/Multiplexer.log" ),

    empty( "" ),

    file_max_buf_len( 1 << 14 ), // 16K
    user_max_buf_len( 1 << 12 ), // 4K
    serv_max_buf_len( 1 << 20 ), // 1M

    vc_con (0),
    vc_sig (0),
    vc_time(0),
    vc_ios (0),
    vc_log (0),
    vc_stat(0),
    vc_mux (0)
{
}

void GAS::init()
{
  vc_con  = new IO_FILE( 1, true,  false );
  vc_sig  = new Signal;               // needs vc_con !!
  vc_time = new TIME_Server;          // needs vc_con, vc_log
  vc_ios  = new IO_Server;            // needs vc_sig, vc_con; vc_time, vc_log, vc_var
  vc_log  = new IO_FILE( file_log, false, false, false, time_log ); // needs vc_ios
  vc_stat = new Stat;                 // needs vc_time
}

GAS::~GAS()
{
  delete vc_stat; vc_stat = 0;
  delete vc_log ; vc_log  = 0;
  delete vc_ios ; vc_ios  = 0;
  delete vc_time; vc_time = 0;
  delete vc_sig ; vc_sig  = 0;
  delete vc_con ; vc_con  = 0;
}

std::unique_ptr<GAS> gas;

//: Actors.C (eof) (c) Igor
