// $Id: Actors.C 15422 2015-11-13 21:06:21Z igor $
// This is a GGS file, licensed under the GPL

/*
    (c) Igor Durdanovic, igord@research.nj.nec.com
    NEC Research Institute
    4 Independence Way
    Princeton, NJ 08540, USA

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
//: Actors.C (bof) (c) Igor Durdanovic

#include "Actors.H"
#include "Signal.H"
#include "Stat.H"
#include "IO_FILE.H"
#include "IO_Server.H"
#include "DB_Server.H"
#include "TIME_Server.H"
#include "TSTAT_Server.H"
#include "VAR_Service.H"
#include "EXE_Service.H"
#include "Message.H"
#include "Client.H"
#include "Game.H"

GAS::GAS()
  : root_level ( 3 ), // this must coresspond to Central definitions
    admin_level ( 2 ),
    user_level ( 1 ),
    unreg_level ( 0 ),

    time_log  ( 24*60*60 ),
    time_db   ( 24*60*60 ),
    time_stat (       60 ),
    time_rank (    10*60 ),
    time_save (    10*60 ),

    file_con     ( "log/" + std::string(RegularBoardGame::GAME_NAME) + ".con" ),
    file_log     ( "log/" + std::string(RegularBoardGame::GAME_NAME) + ".log" ),
    file_db      ( "db/"  + std::string(RegularBoardGame::GAME_NAME) + ".db" ),
    file_save    ( "db/"  + std::string(RegularBoardGame::GAME_NAME) + ".save" ),
    file_archive ( "db/"  + std::string(RegularBoardGame::GAME_NAME) + ".game" ),

    group_td ( "_td" ),

    str_empty( "" ),
    str_plus ( "+" ),
    str_minus( "-" ),
    str_pct  ( "%" ),
    str_star ( "*" ),
    str_quest( "?" ),
    
    login_system  ( RegularBoardGame::LOGIN_SYSTEM ),
    login_service ( RegularBoardGame::LOGIN_SERVICE ),
    passw_service( "" ), // set in main

    file_max_buf_len ( 1 << 14 ), // 16K
    user_max_buf_len ( 1 << 12 ), // 4K
    serv_max_buf_len ( 1 << 20 ), // 1M

    max_ignore ( 64 ),
    max_notify ( 64 ),
    max_track  ( 64 ),
    max_str_len( 1 << 10 ), // 1K

    warn_blocks ( 1024 * 8 ),
    exit_blocks ( 1024 * 2 ),
    warn_inodes ( 1024 * 8 ),
    exit_inodes ( 1024 * 2 ),
    warn_memory ( 1024 * 1024 * 4 ),
    exit_memory ( 1024 * 1024 ),
    svc_ready   ( true ), 
    tstat_ready ( false ),

    vc_tstat(0),
    vc_con  (0),
    vc_sig  (0),
    vc_time (0),
    vc_db   (0),
    vc_save (0),
    vc_ios  (0),
    vc_log  (0),
    vc_var  (0),
    vc_exe  (0),
    vc_stat (0),
    vc_mssg (0)
{
}

void GAS::init()
{
  vc_tstat = new TSTAT_Server;
  vc_con   = new IO_FILE( file_con.c_str(), true,  false );
  vc_sig   = new Signal;               // needs vc_con !!
  vc_time  = new TIME_Server;          // needs vc_con, vc_log
  vc_db    = new DB_Server( file_db.c_str(), IO_Buffer::NONE );   // needs vc_con, vc_log
  vc_save  = new DB_Server( file_save.c_str(), IO_Buffer::NONE ); // needs vc_con, vc_log
  vc_ios   = new IO_Server;               // needs vc_sig, vc_con; vc_time, vc_log, vc_var
  vc_log   = new IO_FILE( file_log.c_str(), false, false, false, time_log ); // needs vc_ios
  vc_var   = new VAR_Service;
  vc_exe   = new EXE_Service;
  vc_stat  = new Stat;
  vc_mssg  = new Message;               // needs nothing
}

GAS::~GAS()
{
  delete vc_mssg;  vc_mssg = 0;
  delete vc_stat;  vc_stat = 0;
  delete vc_exe;   vc_exe  = 0;
  delete vc_var;   vc_var  = 0;
  delete vc_log;   vc_log  = 0;
  delete vc_ios;   vc_ios  = 0;
  delete vc_save;  vc_save = 0;
  delete vc_db;    vc_db   = 0;
  delete vc_time;  vc_time = 0;
  delete vc_sig;   vc_sig  = 0;
  delete vc_con;   vc_con  = 0;
  delete vc_tstat; vc_tstat = 0;
}

std::unique_ptr<GAS> gas;

//: Actors.C (eof) (c) Igor
