// $Id: Actors.H 15422 2015-11-13 21:06:21Z igor $
// This is a GGS file, licensed under the GPL

/*
    (c) Igor Durdanovic, igord@research.nj.nec.com
    NEC Research Institute
    4 Independence Way
    Princeton, NJ 08540, USA

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
//: Actors.H (bof) (c) Igor Durdanovic

#ifndef Actors_H
#define Actors_H

#include "CPP.H"
#include "String.H"
#include <memory>

class Signal;
class Stat;
class IO_FILE;
class IO_Server;
class TIME_Server;
class TSTAT_Server;
class DB_Server;
class VAR_Service;
class EXE_Service;
class Message;

struct GAS
{
  GAS();
  ~GAS();
  void init();
  
  const sint4  root_level;
  const sint4 admin_level;
  const sint4  user_level;
  const sint4 unreg_level;
  
  sint4  time_log;
  sint4  time_db;
  sint4  time_stat;
  sint4  time_rank;
  sint4  time_save; // VAR_Service
  
  const String file_con;
  const String file_log;
  const String file_db;
  const String file_save;
  const String file_archive;
  
  const String group_td;
  
  const String str_empty;
  const String str_plus;
  const String str_minus;
  const String str_pct;
  const String str_star;
  const String str_quest;
  
  const String login_system;
  const String login_service;
  String passw_service; // set in main
  
  const uint4 file_max_buf_len;
  const uint4 user_max_buf_len;
  const uint4 serv_max_buf_len;
  
  const uint4 max_ignore;
  const uint4 max_notify;
  const uint4 max_track;
  const uint4 max_str_len;
  
  const uint4 warn_blocks; // warn, disk space reaching low levels
  const uint4 exit_blocks; // exit, no disk space left
  const uint4 warn_inodes; // warn, file system reaching low levels
  const uint4 exit_inodes; // exit, no file system resources left
  const uint4 warn_memory; // warn if memory gets tight
  const uint4 exit_memory; // exit if memory gets to tight
  /* */ bool  svc_ready;
  bool  tstat_ready; // collect TSTAT
  
  TSTAT_Server*    vc_tstat;
  IO_FILE*         vc_con;
  Signal*          vc_sig;
  TIME_Server*     vc_time;
  DB_Server*       vc_db;
  DB_Server*       vc_save;
  IO_Server*       vc_ios;
  IO_FILE*         vc_log;
  VAR_Service*     vc_var;
  EXE_Service*     vc_exe;
  Stat*            vc_stat;
  Message*         vc_mssg;
};

extern std::unique_ptr<GAS> gas;

#endif

//: Actors.H (eof) (c) Igor
